/* Copyright (c) 2019 Atmark Techno, Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <modbus.h>

#define SERIAL_SPEED 19200
#define SERIAL_PARITY 'N'
#define SERIAL_DATABIT 8
#define SERIAL_STOPBIT 1
#define SLAVE_ADDRESS 3
#define COMMAND_LEN 41
#define MAX_BRIGHTNESS_DATA_LEN 3

int main(int argc, char *argv[])
{
	modbus_t *ctx;
	uint8_t req[MODBUS_RTU_MAX_ADU_LENGTH];
	uint16_t start_addr;
	int len;
	int target_led_number;
	int led_brightness_data;
	int temp;
	char led_command[COMMAND_LEN + MAX_BRIGHTNESS_DATA_LEN];

	if (argc <= 1)
	{
		printf("Usage: %s <device>\n\n", argv[0]);
		return EXIT_SUCCESS;
	}

	/* マスタからの値読み取りまたは書き込み要求を受信した場合に使用する配列を用意する。
	 * 以下の4種類の値の、配列の大きさを引数として渡す。
	 * bit input bit registers inputs registers
	 * 各値は自由に設定可能。
	 */
	modbus_mapping_t *mapping;
	mapping = modbus_mapping_new(0, 1, 30, 2);

	if (!mapping)
	{
		fprintf(stderr, "Failed to allocate the mapping: %s\n", modbus_strerror(errno));
		exit(1);
	}

	/* RTUのlibmodbusコンテキストを作成 */
	ctx = modbus_new_rtu(argv[1], SERIAL_SPEED, SERIAL_PARITY, SERIAL_DATABIT, SERIAL_STOPBIT);
	if (!ctx)
	{
		fprintf(stderr, "Failed to create the context: %s\n", modbus_strerror(errno));
		modbus_mapping_free(mapping);
		exit(1);
	}

	/* modbusの動作を確認できるデバッグ機能 */
	modbus_set_debug(ctx, TRUE);

	if (modbus_connect(ctx) == -1)
	{
		fprintf(stderr, "Unable to connect: %s\n", modbus_strerror(errno));
		modbus_mapping_free(mapping);
		modbus_free(ctx);
		exit(1);
	}

	modbus_set_slave(ctx, SLAVE_ADDRESS);

	while (1)
	{
		len = modbus_receive(ctx, req);
		if (len == -1)
			break;

		if (req[1] != 0x06)
		{
			printf("unsupported function code.\n");
			len = modbus_reply_exception(ctx, req, MODBUS_EXCEPTION_ILLEGAL_FUNCTION);
			continue;
		}

		start_addr = (req[2] << 8) | req[3];

		switch (start_addr)
		{
		case 0x0003:
			target_led_number = 3;
			break;
		case 0x0004:
			target_led_number = 4;
			break;
		case 0x0005:
			target_led_number = 5;
			break;
		default:
			printf("unknown led number. armadillo-iot g3l led number [3 , 4 , 5].\n");
			len = modbus_reply_exception(ctx, req, MODBUS_EXCEPTION_ILLEGAL_DATA_VALUE);
			continue;
		}

		len = modbus_reply(ctx, req, len, mapping);
		if (len == -1)
			break;

		/* mappingの保有レジスタへのデータ保持はmodbus_reply関数内で行われる。
		 * そのため、mappingの保有レジスタのデータチェックはこのタイミングで行う。
		 */
		led_brightness_data = mapping->tab_registers[target_led_number];
		snprintf(led_command, sizeof(led_command), "echo %d > /sys/class/leds/led%d/brightness", led_brightness_data, target_led_number);
		printf("led command : %s\n", led_command);
		system(led_command);
	}
	printf("Exit the loop: %s\n", modbus_strerror(errno));

	modbus_mapping_free(mapping);
	modbus_close(ctx);
	modbus_free(ctx);

	return EXIT_SUCCESS;
}
